#include "system.h"
#include "sys/alt_stdio.h"
#include "sys/alt_sys_wrappers.h"
#include "altera_avalon_spi_regs.h"
#include "altera_avalon_spi.h"

#include "bmp280.h"
#include "bmp280_regs.h"

uint8_t spiWriteData[2];
uint8_t spiReadData[3];

uint16_t dig_T1;
int16_t  dig_T2;
int16_t  dig_T3;

uint16_t dig_P1;
int16_t  dig_P2;
int16_t  dig_P3;
int16_t  dig_P4;
int16_t  dig_P5;
int16_t  dig_P6;
int16_t  dig_P7;
int16_t  dig_P8;
int16_t  dig_P9;

void BMP280SetSPI(void){
	//wybieram aktywowanie ukadu, ktrego pin CS podczony jest do wyjcia [0]
	IOWR_ALTERA_AVALON_SPI_SLAVE_SEL(SPI_BASE, 0b1);
	//teraz aktywuj na ten ukad na ~1ms
	//pozwalam sobie na nadpisanie wartoci rejestru,
	//bo jestem pewny, e nie korzystam z innych jego bitw
	IOWR_ALTERA_AVALON_SPI_CONTROL(SPI_BASE, ALTERA_AVALON_SPI_CONTROL_SSO_MSK);
	ALT_USLEEP(1000);
	IOWR_ALTERA_AVALON_SPI_CONTROL(SPI_BASE, 0);
}

uint16_t BMP280Read16U(uint8_t address){
	spiWriteData[0] = address & BMP280_SPI_R;
	alt_avalon_spi_command(SPI_BASE, 0, 1, spiWriteData, 2, spiReadData, 0);
	return (((uint16_t)spiReadData[0])<<0) | (((uint16_t)spiReadData[1])<<8);
}

int16_t BMP280Read16S(uint8_t address){
	return (int16_t)BMP280Read16U(address);
}

void BMP280Init(void){
	spiWriteData[0] = BMP280_CTRL_MEAS & BMP280_SPI_W;
	spiWriteData[1] = BMP280_MODE_NORMAL | BMP280_OSRS_P_16 | BMP280_OSRS_T_16;
	alt_avalon_spi_command(SPI_BASE, 0, 2, spiWriteData, 0, spiReadData, 0);

	dig_T1 = BMP280Read16U(BMP280_DIG_T1);
	dig_T2 = BMP280Read16S(BMP280_DIG_T2);
	dig_T3 = BMP280Read16S(BMP280_DIG_T3);

	dig_P1 = BMP280Read16U(BMP280_DIG_P1);
	dig_P2 = BMP280Read16S(BMP280_DIG_P2);
	dig_P3 = BMP280Read16S(BMP280_DIG_P3);
	dig_P4 = BMP280Read16S(BMP280_DIG_P4);
	dig_P5 = BMP280Read16S(BMP280_DIG_P5);
	dig_P6 = BMP280Read16S(BMP280_DIG_P6);
	dig_P7 = BMP280Read16S(BMP280_DIG_P7);
	dig_P8 = BMP280Read16S(BMP280_DIG_P8);
	dig_P9 = BMP280Read16S(BMP280_DIG_P9);

}

uint32_t BMP280ReadT(void){
	spiWriteData[0] = BMP280_TEMP & BMP280_SPI_R;
	alt_avalon_spi_command(SPI_BASE, 0, 1, spiWriteData, 3, spiReadData, 0);
	return (((uint32_t)spiReadData[0])<<12) | (((uint32_t)spiReadData[1])<<4) | (((uint32_t)spiReadData[2])>>4);
}

uint32_t BMP280ReadP(void){
	spiWriteData[0] = BMP280_PRESS & BMP280_SPI_R;
	alt_avalon_spi_command(SPI_BASE, 0, 1, spiWriteData, 3, spiReadData, 0);
	return (((uint32_t)spiReadData[0])<<12) | (((uint32_t)spiReadData[1])<<4) | (((uint32_t)spiReadData[2])>>4);
}

int32_t t_fine;
// jednostka to 0.01 st. C (czyli 2018 to 20.18 st. C)
int32_t BMP280ConvertT(int32_t adc_T){
	int32_t var1, var2;

	var1  = ((((adc_T>>3) - ((int32_t)dig_T1 <<1))) *
	   ((int32_t)dig_T2)) >> 11;

	var2  = (((((adc_T>>4) - ((int32_t)dig_T1)) *
		 ((adc_T>>4) - ((int32_t)dig_T1))) >> 12) *
	   ((int32_t)dig_T3)) >> 14;

	t_fine = var1 + var2;

	return (t_fine * 5 + 128) >> 8;
}

// jednostka to 0.001 hPa (czyli 1013123 to 1013.123 hPa)
// przed wywoaniem tej funkcji trzeba obligatoryjnie wywoac funkcj konwersji temperatury
uint32_t BMP280ConvertP(int32_t adc_P){
	int64_t var1, var2, p;

	var1 = ((int64_t)t_fine) - 128000;
	var2 = var1 * var1 * (int64_t)dig_P6;
	var2 = var2 + ((var1*(int64_t)dig_P5)<<17);
	var2 = var2 + (((int64_t)dig_P4)<<35);
	var1 = ((var1 * var1 * (int64_t)dig_P3)>>8) +
	((var1 * (int64_t)dig_P2)<<12);
	var1 = (((((int64_t)1)<<47)+var1))*((int64_t)dig_P1)>>33;

	if (var1 == 0) {
		return 0;  // avoid exception caused by division by zero
	}
	p = 1048576 - adc_P;
	p = (((p<<31) - var2)*3125) / var1;
	var1 = (((int64_t)dig_P9) * (p>>13) * (p>>13)) >> 25;
	var2 = (((int64_t)dig_P8) * p) >> 19;

	p = (((p + var1 + var2) >> 8) + (((int64_t)dig_P7)<<4)) * 10ULL / 256ULL;

	return (uint32_t)p;
}
